"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var t = require('@babel/types');

var findUp = require('find-up');

var fs = require('fs');

var path = require('path');

var template = require('@babel/template').default;

function formatDate(date, fmt) {
  var o = {
    'M+': date.getMonth() + 1,
    'd+': date.getDate(),
    'h+': date.getHours(),
    'm+': date.getMinutes(),
    's+': date.getSeconds(),
    'q+': Math.floor((date.getMonth() + 3) / 3),
    S: date.getMilliseconds()
  };
  if (/(y+)/.test(fmt)) fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
  Object.keys(o).forEach(function (k) {
    if (new RegExp('(' + k + ')').test(fmt)) {
      fmt = fmt.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ('00' + o[k]).substr(('' + o[k]).length));
    }
  });
  return fmt;
}

;

function fileExists(path) {
  try {
    return !fs.accessSync(path, fs.F_OK);
  } catch (e) {
    return false;
  }
}

function getConfigPath(filename) {
  var packagePath = findUp.sync('package.json', {
    cwd: path.dirname(filename),
    type: 'file'
  });
  if (packagePath && fileExists(packagePath)) return packagePath;
}

var NOW = formatDate(new Date(), 'yyyy-MM-dd hh:mm:ss');
var constCached = [];

function getConstCache(filename) {
  var pkgPath = getConfigPath(filename);
  if (!pkgPath || filename === path.resolve(pkgPath)) return {};
  var cwd;
  var cache = constCached[pkgPath];

  if (!cache) {
    var pkg = require(pkgPath);

    if (!pkg || !Object.keys(pkg).length) return {};
    cwd = path.dirname(pkgPath);
    cache = {
      pkg: pkg,
      cwd: cwd
    };
    constCached[pkgPath] = cache;
    cache.now = NOW;
  }

  cwd = cache.cwd;
  cache.filename = '/' + path.relative(cwd, filename).replace(/\\/g, '/');
  cache.dirname = '/' + path.relative(cwd, path.dirname(filename)).replace(/\\/g, '/');
  return cache;
}

function arr2Expression(arr, parent) {
  var temp = '';
  var vars = {};
  arr.forEach(function (v, i) {
    var expr = var2Expression(v, arr);
    if (!expr) return;
    var key = "$".concat(i);
    temp += (temp ? ', ' : '') + key;
    vars[key] = expr;
  });
  return template("[".concat(temp, "]"))(vars);
}

function obj2Expression(obj, parent) {
  var props = Object.keys(obj).map(function (k) {
    var v = obj[k];
    var expr = var2Expression(v, obj);
    if (!expr) return;
    return t.objectProperty(t.identifier(k), expr);
  }).filter(Boolean);
  return t.objectExpression(props);
}

function var2Expression(v, parent) {
  if (t.isNode(v)) return v;
  if (v === undefined) return;
  if (Array.isArray(v)) return arr2Expression(v, parent);

  switch (_typeof(v)) {
    case 'string':
      return t.stringLiteral(v);

    case 'boolean':
      return t.booleanLiteral(v);

    case 'number':
      return t.numericLiteral(v);

    case 'object':
      if (v === null) return t.nullLiteral();
      if (v instanceof RegExp) return t.regExpLiteral(v.source, v.flags);
      if (v instanceof Date) return template('new Date(TIME)')({
        TIME: t.numericLiteral(v.getTime())
      });
      if (v instanceof Function) return template(v.toString())();
      return obj2Expression(v, parent);

    default:
      return t.identifier('undefined');
  }
}

module.exports = {
  getConstCache: getConstCache,
  var2Expression: var2Expression,
  arr2Expression: arr2Expression,
  obj2Expression: obj2Expression
};